/**
can.c

Routines for sending and receiving messages for configuration and/or
communication over CAN network using a SJA1000 transceiver.
For use in UL_USB1 module, it runs in Intel mode.
See documentation for details.

*/


#include "can/can.h"

/// Baud rates
#if SJA1000_CLK==(24000000)
	/*	Bus speed, Precaler, SJW, TSEG1, TSEG2
	 *	For SJW setting we assume 1% xtal accuracy
	 */
	const long sja1000_freqs[3][5]=
	{{1000,	0x00,	0xC0,	0x08,	0x10}
	,{250,	0x02,	0xC0,	0x0A,	0x30}
	,{100,	0x07,	0xC0,	0x09,	0x30}};
	const int sja1000_freq_cnt=3;
#endif

struct can_baudparams_t canbaud;

void can_comm_init()
{
	// Due to change in design there is CS_PIN connected with ALE_PIN and ALE_PIN connection to LPC is interrupted
	// We don't use ALE_PIN
	//IO0DIR|=P0_SJA1000_ALE_PIN|P0_SJA1000_CS_PIN|P0_SJA1000_RD_PIN|P0_SJA1000_WR_PIN;
	IO0DIR|=P0_SJA1000_CS_PIN|P0_SJA1000_RD_PIN|P0_SJA1000_WR_PIN;
	IO0DIR&=~(P0_SJA1000_INT_PIN);

	SET_OUT_PIN(IO0,P0_SJA1000_CS_PIN);
	CLR_OUT_PIN(IO1,P1_SJA1000_RST_PIN);
	SJA1000_INIT_DELAY();
	SET_OUT_PIN(IO0,P0_SJA1000_CS_PIN);
	// Due to change in design there is CS_PIN connected with ALE_PIN and ALE_PIN connection to LPC is interrupted
	//	CLR_OUT_PIN(IO0,P0_SJA1000_ALE_PIN);
	SET_OUT_PIN(IO1,P1_SJA1000_RST_PIN);
	SJA1000_INIT_DELAY();
}

int can_write(uint8_t address,uint8_t* data)
{
	IO1DIR|=0x00FF0000; // Port as output to send data
	IO1CLR=0x00FF0000; // Clear all data on port
	// Init
	SET_OUT_PIN(IO0,P0_SJA1000_RD_PIN);	// Stays high on write
	SET_OUT_PIN(IO0,P0_SJA1000_WR_PIN); // Stays high on address write
	SET_OUT_PIN(IO0,P0_SJA1000_CS_PIN); // Sets output buffers to third state
	SJA1000_DELAY();
	//SET_OUT_PIN(IO0,P0_SJA1000_ALE_PIN); // Start command

	// Set memory address
	IO1SET=__val2mfld(0x00FF0000,address); // Shift data to SJA pins and output them
	SJA1000_DELAY();
	//CLR_OUT_PIN(IO0,P0_SJA1000_ALE_PIN); // Makes address active
	CLR_OUT_PIN(IO0,P0_SJA1000_CS_PIN);
	SJA1000_DELAY();

	// Setting data
	CLR_OUT_PIN(IO0,P0_SJA1000_WR_PIN);

	IO1CLR=0x00FF0000;
	IO1SET=__val2mfld(0x00FF0000,*data);
	SJA1000_DELAY();
	SET_OUT_PIN(IO0,P0_SJA1000_WR_PIN); // Data should be accepted by now
	SET_OUT_PIN(IO0,P0_SJA1000_CS_PIN);
	SJA1000_DELAY();
	return 0;
}

int can_read(const uint8_t address,uint8_t* data)
{
	IO1DIR|=0x00FF0000; // Port as output to set address
	IO1CLR=0x00FF0000; // Clear all data
	// Init
	SET_OUT_PIN(IO0,P0_SJA1000_WR_PIN); // Stays high on read
	SET_OUT_PIN(IO0,P0_SJA1000_RD_PIN); // Stays high while entering address
	SET_OUT_PIN(IO0,P0_SJA1000_CS_PIN);
	SJA1000_DELAY();
	//SET_OUT_PIN(IO0,P0_SJA1000_ALE_PIN);

	// Request memory address
	IO1SET=__val2mfld(0x00FF0000,address);
	SJA1000_DELAY();
	//CLR_OUT_PIN(IO0,P0_SJA1000_ALE_PIN);
	CLR_OUT_PIN(IO0,P0_SJA1000_CS_PIN);

	// Get data

	IO1DIR&=~0x00FF0000; // Sets port as input
	CLR_OUT_PIN(IO0,P0_SJA1000_RD_PIN);
	SJA1000_DELAY();
	*data=__mfld2val(0x00FF0000,IO1PIN);
	SET_OUT_PIN(IO0,P0_SJA1000_RD_PIN);
	SET_OUT_PIN(IO0,P0_SJA1000_CS_PIN);
	SJA1000_DELAY();
	return 0;
}

int can_init(){
	uint8_t data=0,count=0;
	do {
 		can_comm_init();
		can_read(SJAMOD,&data);
		if (count++ > 50)
			return -1;
	} while (!(data&sjaMOD_RM));

	data=sjaCDR_CLKOUT_DIV1|sjaCDR_CLK_OFF|sjaCDR_CBP|sjaCDR_PELICAN;
	can_write((uint8_t)SJACDR,(uint8_t*)&data);

	// Single acceptance filter, reset mode
	data=sjaMOD_AFM|sjaMOD_RM;
	can_write((uint8_t)SJAMOD,(uint8_t*)&data);

	// Enabling all interrupt sources
	data=sjaENABLE_INTERRUPTS;
	can_write((uint8_t)SJAIER,(uint8_t*)&data);

	// Accept all messages
	data=0xFF;
	can_write((uint8_t)SJAAMR0,(uint8_t*)&data);
	can_write((uint8_t)SJAAMR0+1,(uint8_t*)&data);
	can_write((uint8_t)SJAAMR0+2,(uint8_t*)&data);
	can_write((uint8_t)SJAAMR0+3,(uint8_t*)&data);

	data=sjaCMR_CDO;
	can_write((uint8_t)SJACMR,(uint8_t*)&data);

	return 0;
}
